# Generated by Django 3.1.5 on 2021-01-31 19:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0013_update_report_type_choices"),
    ]

    operations = [
        migrations.AlterField(
            model_name="check",
            name="result",
            field=models.CharField(
                blank=True,
                choices=[
                    ("clear", "Clear"),
                    ("consider", "Consider"),
                    ("unidentified", "Unidentified"),
                ],
                help_text="The final result of the check / reports (from API).",
                max_length=20,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="check",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("awaiting_applicant", "Awaiting applicant"),
                    ("awaiting_approval", "Awaiting approval"),
                    ("awaiting_data", "Awaiting data"),
                    ("cancelled", "Cancelled"),
                    ("complete", "Complete"),
                    ("in_progress", "In progress"),
                    ("paused", "Paused"),
                    ("reopened", "Reopened"),
                    ("withdrawn", "Withdrawn"),
                ],
                db_index=True,
                help_text="The current state of the check / report (from API).",
                max_length=20,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="report",
            name="result",
            field=models.CharField(
                blank=True,
                choices=[
                    ("clear", "Clear"),
                    ("consider", "Consider"),
                    ("unidentified", "Unidentified"),
                ],
                help_text="The final result of the check / reports (from API).",
                max_length=20,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="report",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("awaiting_applicant", "Awaiting applicant"),
                    ("awaiting_approval", "Awaiting approval"),
                    ("awaiting_data", "Awaiting data"),
                    ("cancelled", "Cancelled"),
                    ("complete", "Complete"),
                    ("in_progress", "In progress"),
                    ("paused", "Paused"),
                    ("reopened", "Reopened"),
                    ("withdrawn", "Withdrawn"),
                ],
                db_index=True,
                help_text="The current state of the check / report (from API).",
                max_length=20,
                null=True,
            ),
        ),
    ]



from __future__ import annotations

import logging

from dateutil.parser import parse as date_parse
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

logger = logging.getLogger(__name__)


class Event(models.Model):
    """Used to record callback events received from the API."""

    onfido_id = models.CharField(
        "Onfido ID",
        max_length=40,
        help_text=_("The Onfido ID of the related resource."),
    )
    resource_type = models.CharField(
        max_length=20, help_text=_("The resource_type returned from the API callback.")
    )
    action = models.CharField(
        max_length=20, help_text=_("The event name as returned from the API callback.")
    )
    status = models.CharField(
        max_length=20, help_text=_("The status of the object after the event.")
    )
    completed_at = models.DateTimeField(
        help_text=_("The timestamp returned from the Onfido API."),
        blank=True,
        null=True,
    )
    received_at = models.DateTimeField(
        help_text=_("The timestamp when the server received the event."),
    )
    raw = models.JSONField(
        help_text=_("The raw JSON returned from the API."), blank=True, null=True
    )

    class Meta:
        ordering = ["completed_at"]

    def __str__(self) -> str:
        return "{} event occurred on {}.{}".format(
            self.action, self.resource_type, self.onfido_id
        )

    def __repr__(self) -> str:
        return "<Event id={} action='{}' onfido_id='{}.{}'>".format(
            self.id, self.action, self.resource_type, self.onfido_id
        )

    def _resource_manager(self) -> models.Manager:
        """Return the appropriate model manager for the resource_type."""
        if self.resource_type not in (
            "check",
            "report",
        ):
            raise ValueError(f"Unknown resource type: {self.resource_type}")
        if self.resource_type == "check":
            from .check import Check

            return Check.objects
        elif self.resource_type == "report":
            from .report import Report

            return Report.objects

    @property
    def resource(self) -> models.Manager:
        """Return the underlying Check or Report resource."""
        return self._resource_manager().get(onfido_id=self.onfido_id)

    @property
    def user(self) -> settings.AUTH_USER_MODEL:
        """Return the user to whom the resource refers."""
        return self.resource.user

    def parse(self, raw_json: dict) -> Event:
        """Parse the raw value out into other properties."""
        self.raw = raw_json
        payload = self.raw["payload"]
        self.resource_type = payload["resource_type"]
        self.action = payload["action"]
        obj = payload["object"]
        self.onfido_id = obj["id"]
        self.status = obj["status"]
        self.completed_at = date_parse(obj["completed_at_iso8601"])
        return self



# Generated by Django 3.1.3 on 2021-03-04 08:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0016_update_applicant_user_field"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="applicant",
            options={},
        ),
    ]



"""
Basic wire operations with the API - GET/POST/PUT.

This is a simple wrapper around requests.

"""
from __future__ import annotations

import logging
from urllib import parse as urlparse

import requests
from django.http import HttpResponse

from .settings import API_KEY

logger = logging.getLogger(__name__)

# the API HTTP root url
API_ROOT = "https://api.onfido.com/v3/"


class ApiError(Exception):
    """Error raised when interacting with the API."""

    def __init__(self, response: HttpResponse) -> None:
        """Initialise error from response object."""
        data = response.json()
        logger.debug("Onfido API error: {}".format(data))
        super().__init__(data["error"]["message"])
        self.error_type = data["error"]["type"]


def _url(path: str) -> str:
    """Format absolute API URL."""
    return urlparse.urljoin(API_ROOT, path)


def _headers(api_key: str = API_KEY) -> dict[str, str]:
    """Format request headers."""
    return {
        "Authorization": "Token token={}".format(api_key),
        "Content-Type": "application/json",
    }


def _respond(response: HttpResponse) -> dict:
    """Process common response object."""
    if not str(response.status_code).startswith("2"):
        raise ApiError(response)
    data = response.json()
    logger.debug("Onfido API response: %s", data)
    return data


def get(href: str) -> dict:
    """Make a GET request and return the response as JSON."""
    logger.debug("Onfido API GET request: %s", href)
    return _respond(requests.get(_url(href), headers=_headers()))


def post(href: str, data: dict) -> dict:
    """Make a POST request and return the response as JSON."""
    logger.debug("Onfido API POST request: %s: %s", href, data)
    return _respond(requests.post(_url(href), headers=_headers(), json=data))



from __future__ import annotations

from typing import TYPE_CHECKING

import simplejson as json  # simplejson supports Decimal
from django.contrib import admin
from django.db import models
from django.http import HttpRequest
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from .models import Applicant, Check, Event, Report

if TYPE_CHECKING:
    from .models.base import BaseModel


class ResultMixin(object):
    """Adds custom action for overriding is_clear."""

    def mark_as_clear(self, request: HttpRequest, queryset: models.QuerySet) -> None:
        """Call mark_as_clear on all objects in the queryset."""
        for obj in queryset:
            obj.mark_as_clear(request.user)

    mark_as_clear.short_description = _("Mark selected items as clear")  # type: ignore


class EventsMixin(object):
    """Pretty print Events relating to an object."""

    def _events(self, obj: Event) -> str:
        """Pretty print object events."""
        events = obj.events()
        html = "".join(
            ["<li>{}: {}</li>".format(e.completed_at.date(), e.action) for e in events]
        )
        return mark_safe("<ul>{}</ul>".format(html))  # noqa: S703, S308

    _events.short_description = _("Related events")  # type: ignore


class RawMixin(object):
    """Admin mixin used to pprint raw JSON fields."""

    def _raw(self, obj: BaseModel) -> str:
        """
        Return an indented HTML pretty-print version of JSON.

        Take the event_payload JSON, indent it, order the keys and then
        present it as a <code> block. That's about as good as we can get
        until someone builds a custom syntax function.

        """
        pretty = json.dumps(obj.raw, sort_keys=True, indent=4, separators=(",", ": "))
        html = pretty.replace(" ", "&nbsp;").replace("\n", "<br>")
        return mark_safe("<code>{}</code>".format(html))  # noqa: S703, S308

    _raw.short_description = _("Raw (from API)")  # type: ignore


class UserMixin(object):
    """Admin mixin used to add _user function."""

    def _user(self, obj: Applicant | Check | Report) -> str:
        """
        Return user's real name.

        It's impossible to create an Applicant in the Onfido API
        with a blank first/last name, so assuming that get_full_name
        will not be blank.

        """
        return obj.user.get_full_name().title()

    _user.short_description = "User"  # type: ignore


class ApplicantAdmin(RawMixin, UserMixin, admin.ModelAdmin):
    """Admin model for Applicant objects."""

    list_display = ("onfido_id", "_user", "created_at")
    list_filter = ("created_at",)
    ordering = ("user__first_name", "user__last_name", "user__username")
    readonly_fields = ("onfido_id", "user", "created_at", "_raw")
    search_fields = ("onfido_id", "user__first_name", "user__last_name")
    raw_id_fields = ("user",)
    exclude = ("raw",)


admin.site.register(Applicant, ApplicantAdmin)


class CheckAdmin(ResultMixin, EventsMixin, RawMixin, UserMixin, admin.ModelAdmin):
    """Admin model for Check objects."""

    list_display = (
        "onfido_id",
        "_user",
        "status",
        "result",
        "created_at",
        "updated_at",
        "is_clear",
    )
    readonly_fields = (
        "onfido_id",
        "user",
        "created_at",
        "applicant",
        "status",
        "result",
        "updated_at",
        "_raw",
        "_events",
    )
    search_fields = ("onfido_id", "user__first_name", "user__last_name")
    list_filter = ("created_at", "updated_at", "status", "result")
    ordering = ("user__first_name", "user__last_name")
    raw_id_fields = ("applicant", "user")
    exclude = ("raw",)
    actions = ("mark_as_clear",)


admin.site.register(Check, CheckAdmin)


class ReportAdmin(ResultMixin, EventsMixin, RawMixin, UserMixin, admin.ModelAdmin):
    """Admin model for Report objects."""

    list_display = (
        "onfido_id",
        "_user",
        "report_type",
        "status",
        "result",
        "created_at",
        "updated_at",
        "is_clear",
    )
    ordering = ("user__first_name", "user__last_name")
    readonly_fields = (
        "onfido_id",
        "user",
        "onfido_check",
        "report_type",
        "status",
        "result",
        "created_at",
        "updated_at",
        "_raw",
    )
    search_fields = (
        "onfido_id",
        "onfido_check__onfido_id",
        "user__first_name",
        "user__last_name",
    )
    list_filter = ("created_at", "updated_at", "report_type", "status", "result")
    raw_id_fields = ("onfido_check", "user")
    exclude = ("raw",)
    actions = ("mark_as_clear",)


admin.site.register(Report, ReportAdmin)


class EventAdmin(RawMixin, UserMixin, admin.ModelAdmin):
    """Admin model for Event objects."""

    list_display = (
        "onfido_id",
        "resource_type",
        "_user",
        "action",
        "status",
        "completed_at",
    )
    list_filter = ("action", "resource_type", "status", "completed_at")
    readonly_fields = (
        "onfido_id",
        "resource_type",
        "action",
        "status",
        "completed_at",
        "received_at",
        "_raw",
    )
    search_fields = ("onfido_id",)
    exclude = ("raw",)


admin.site.register(Event, EventAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0002_event"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="action",
            field=models.CharField(
                help_text="The event name as returned from the API callback.",
                max_length=20,
            ),
        ),
    ]



# Generated by Django 3.1.3 on 2021-05-05 11:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0017_auto_20210304_0241"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="check",
            name="is_clear",
        ),
        migrations.RemoveField(
            model_name="report",
            name="is_clear",
        ),
    ]



from __future__ import annotations

from typing import Any, Iterable

from django.conf import settings

from .api import get, post
from .models import Applicant, Check, Report


def create_applicant(user: settings.AUTH_USER_MODEL, **kwargs: Any) -> Applicant:
    """
    Create an applicant in the Onfido system.

    Args:
        user: a Django User instance to register as an applicant.
    Kwargs:
       any kwargs passed in are merged into the data dict sent to the API. This
       enables support for additional applicant properties - e.g dob, gender,
       country, and any others that may change over time.
       See https://documentation.onfido.com/#create-applicant for details.
    """
    data = {
        "first_name": user.first_name,
        "last_name": user.last_name,
        "email": user.email,
    }
    data.update(kwargs)
    response = post("applicants", data)
    return Applicant.objects.create_applicant(user, response)


def create_check(applicant: Applicant, report_names: Iterable, **kwargs: Any) -> Check:
    """
    Create a new Check (and child Reports).

    Args:
        applicant: Applicant for whom the checks are being made.
        report_names: list of strings, each of which is a valid report type.

    Kwargs:
        any kwargs passed in are merged into the data dict sent to the API. This
        enables support for additional check properties - e.g. redirect_uri,
        tags, suppress_form_emails and any other that may change over time. See
        https://documentation.onfido.com/#checks for details.

    Returns a new Check object, and creates the child Report objects.

    """
    data = {
        "applicant_id": applicant.onfido_id,
        "report_names": report_names,
    }
    # merge in the additional kwargs
    data.update(kwargs)
    response = post("checks", data)
    check = Check.objects.create_check(applicant=applicant, raw=response)
    reports = get(f"reports?check_id={check.onfido_id}")
    for report in reports["reports"]:
        Report.objects.create_report(check=check, raw=report)
    return check



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0006_rename_event_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="onfido_id",
            field=models.CharField(
                help_text="The Onfido ID of the related resource.",
                max_length=40,
                verbose_name="Onfido ID",
            ),
        ),
        migrations.AlterField(
            model_name="applicant",
            name="onfido_id",
            field=models.CharField(
                help_text="The id returned from the Onfido API.",
                unique=True,
                max_length=40,
                verbose_name="Onfido ID",
            ),
        ),
        migrations.AlterField(
            model_name="check",
            name="onfido_id",
            field=models.CharField(
                help_text="The id returned from the Onfido API.",
                unique=True,
                max_length=40,
                verbose_name="Onfido ID",
            ),
        ),
        migrations.AlterField(
            model_name="report",
            name="onfido_id",
            field=models.CharField(
                help_text="The id returned from the Onfido API.",
                unique=True,
                max_length=40,
                verbose_name="Onfido ID",
            ),
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-08 16:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("onfido", "0014_update_status_result_choices"),
    ]

    operations = [
        migrations.AlterField(
            model_name="check",
            name="check_type",
            field=models.CharField(
                blank=True,
                help_text="DEPRECATED: Check objects no longer support a type.",
                max_length=10,
                null=True,
            ),
        ),
        migrations.RenameField(
            model_name="check",
            old_name="check_type",
            new_name="x_check_type",
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-10 10:12

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("onfido", "0015_update_check_type_field"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="applicant",
            options={"ordering": ["created_at"]},
        ),
        migrations.AlterField(
            model_name="applicant",
            name="user",
            field=models.ForeignKey(
                help_text="Django user that maps to this applicant.",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="onfido_applicants",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from os import getenv

from django.conf import settings


def _setting(key, default):
    return getenv(key, default) or getattr(settings, key, default)


# API key from evnironment by default
API_KEY = _setting("ONFIDO_API_KEY", None)

# Webhook token - see https://documentation.onfido.com/#webhooks
WEBHOOK_TOKEN = _setting("ONFIDO_WEBHOOK_TOKEN", None)
# token must be a bytestring for HMAC function to work
WEBHOOK_TOKEN = str.encode(WEBHOOK_TOKEN) if WEBHOOK_TOKEN else None

# Set to False to turn off event logging
LOG_EVENTS = _setting("ONFIDO_LOG_EVENTS", True)

# Set to True to bypass request verification (NOT RECOMMENDED)
TEST_MODE = _setting("ONFIDO_TEST_MODE", False)


def DEFAULT_REPORT_SCRUBBER(raw):
    """Remove breakdown and properties."""
    return {k: v for k, v in raw.items() if k not in ("breakdown", "properties")}


def DEFAULT_APPLICANT_SCRUBBER(raw):
    """Remove all personal data."""
    return {k: v for k, v in raw.items() if k in ("id", "href", "created_at")}


# functions used to scrub sensitive data from reports
scrub_report_data = (
    getattr(settings, "ONFIDO_REPORT_SCRUBBER", None) or DEFAULT_REPORT_SCRUBBER
)

scrub_applicant_data = (
    getattr(settings, "ONFIDO_APPLICANT_SCRUBBER", None) or DEFAULT_APPLICANT_SCRUBBER
)



from __future__ import annotations

import json


def get_countries() -> list:
    with open("onfido/data/supported_applicant_countries.json") as f:
        countries = json.load(f)
    return countries


def check_supported_country(alpha3: str) -> bool:
    with open("onfido/data/supported_applicant_countries.json") as f:
        countries = json.load(f)

    if alpha3:
        pick_country = next(
            (country for country in countries if country["alpha3"] == alpha3), None
        )
        if pick_country:
            return pick_country["supported_identity_report"]

    return False
